@@ -3,6 +3,7 @@
 
 from __future__ import annotations
 
+import datetime
 import sys
 
 from dxtbx.format.FormatCBFMiniEiger import FormatCBFMiniEiger
@@ -19,11 +20,21 @@ class FormatCBFMiniEigerPetraP14(FormatCBFMiniEiger):
 
         header = FormatCBFMiniEiger.get_cbf_header(image_file)
 
+        # Valid from 22nd May 2021
+        expected_serial = "E-32-0129"
+        if timestamp := FormatCBFMiniEiger._get_timestamp_from_raw_header(header):
+            # We have a timestamp. Let's see what detector we should expect
+
+            # Before 22nd May 2021
+            if timestamp < datetime.datetime(2021, 5, 22):
+                expected_serial = "E-32-0107"
+
+        # Find the line recording detector serial, and check
         for record in header.split("\n"):
             if (
                 "# detector" in record.lower()
                 and "eiger" in record.lower()
-                and "E-32-0107" in record
+                and expected_serial in record
             ):
                 return True
 
