@@ -11,9 +11,9 @@
 #ifndef DXTBX_MODEL_PANEL_H
 #define DXTBX_MODEL_PANEL_H
 
+#include <memory>
 #include <string>
 #include <iostream>
-#include <boost/shared_ptr.hpp>
 #include <scitbx/array_family/shared.h>
 #include <scitbx/array_family/versa.h>
 #include <scitbx/array_family/accessors/c_grid.h>
@@ -28,7 +28,6 @@
 
 namespace dxtbx { namespace model {
 
-  using boost::shared_ptr;
   using scitbx::af::double4;
   using scitbx::af::int2;
   using scitbx::af::int4;
@@ -97,7 +96,7 @@ namespace dxtbx { namespace model {
           tiny<double, 2> trusted_range,
           double thickness,
           std::string material,
-          shared_ptr<PxMmStrategy> convert_coord,
+          std::shared_ptr<PxMmStrategy> convert_coord,
           double mu,
           std::string identifier)
         : PanelData(type,
@@ -155,12 +154,12 @@ namespace dxtbx { namespace model {
     }
 
     /** Get the pixel to millimetre strategy */
-    shared_ptr<PxMmStrategy> get_px_mm_strategy() const {
+    std::shared_ptr<PxMmStrategy> get_px_mm_strategy() const {
       return convert_coord_;
     }
 
     /** Set the pixel to millimetre strategy */
-    void set_px_mm_strategy(shared_ptr<PxMmStrategy> strategy) {
+    void set_px_mm_strategy(std::shared_ptr<PxMmStrategy> strategy) {
       convert_coord_ = strategy;
     }
 
@@ -172,7 +171,7 @@ namespace dxtbx { namespace model {
 
     /** Check the value is valid */
     bool is_value_in_trusted_range(double value) const {
-      return (trusted_range_[0] <= value && value < trusted_range_[1]);
+      return (trusted_range_[0] <= value && value <= trusted_range_[1]);
     }
 
     /** Check the coordinate is valid */
@@ -388,7 +387,7 @@ namespace dxtbx { namespace model {
     }
 
     /**
-     * Apply a mask with the trusted range
+     * Apply a mask with the trusted range: [min_trusted_value, max_trusted_value]
      */
     template <typename T>
     void apply_trusted_range_mask(
@@ -399,12 +398,12 @@ namespace dxtbx { namespace model {
       DXTBX_ASSERT(data.accessor().all_eq(mask.accessor()));
       for (std::size_t i = 0; i < mask.size(); ++i) {
         mask[i] =
-          mask[i] && (trusted_range_[0] < data[i] && data[i] < trusted_range_[1]);
+          mask[i] && (trusted_range_[0] <= data[i] && data[i] <= trusted_range_[1]);
       }
     }
 
     /**
-     * Apply a mask with the trusted range
+     * Apply a mask with the trusted range: [min_trusted_value, max_trusted_value]
      */
     template <typename T>
     scitbx::af::versa<bool, scitbx::af::c_grid<2> > get_trusted_range_mask(
@@ -462,7 +461,7 @@ namespace dxtbx { namespace model {
   protected:
     double gain_;
     double pedestal_;
-    shared_ptr<PxMmStrategy> convert_coord_;
+    std::shared_ptr<PxMmStrategy> convert_coord_;
     std::string identifier_;
     boost::optional<Projection2D> projection_2d_;
   };
