@@ -1,8 +1,12 @@
 from __future__ import annotations
 
+import importlib.metadata
 import logging
 
-import pkg_resources
+try:
+    profile_entry_points = importlib.metadata.entry_points()["dxtbx.profile_model"]
+except KeyError:
+    profile_entry_points = []
 
 
 class ProfileModelFactory:
@@ -17,7 +21,7 @@ class ProfileModelFactory:
         """
         if obj is None:
             return None
-        for entry_point in pkg_resources.iter_entry_points("dxtbx.profile_model"):
+        for entry_point in profile_entry_points:
             if entry_point.name == obj["__id__"]:
                 return entry_point.load().from_dict(obj)
         logging.getLogger("dxtbx.model.profile").warn(
