@@ -41,7 +41,7 @@ def test_single_file_indices(indices, expected_call_count, lazy, dials_data):
         )
         format_class = dxtbx.format.Registry.get_format_class_for_file(filename)
         iset = format_class.get_imageset(
-            [filename], single_file_indices=indices, lazy=lazy
+            [filename], single_file_indices=indices, format_kwargs={"lazy": lazy}
         )
         assert obj.call_count == expected_call_count
         iset.reader().nullify_format_instance()
@@ -387,6 +387,11 @@ class TestImageSequence:
         with pytest.raises(RuntimeError):
             _ = sequence2[5]
 
+        # Check data access matches expected images from the slice
+        panel_data1 = sequence[3][0]
+        panel_data2 = sequence2[0][0]
+        assert panel_data1.all_eq(panel_data2)
+
         assert len(sequence2) == 4
         assert_can_get_detectorbase(sequence2, range(0, 4), 5)
         self.tst_get_models(sequence2, range(0, 4), 5)
@@ -396,11 +401,12 @@ class TestImageSequence:
         with pytest.raises(IndexError):
             _ = sequence[3:7:2]
 
+        # Batch offset should not affect slicing of imagesequence
         # Simulate a scan starting from image 0
         sequence_ = copy.deepcopy(sequence)
         sequence_.get_scan().set_batch_offset(-1)
         sequence3 = sequence_[3:7]
-        assert sequence3.get_array_range() == (4, 8)
+        assert sequence3.get_array_range() == (3, 7)
 
     @staticmethod
     def tst_paths(sequence, filenames1):
@@ -501,7 +507,7 @@ def test_SACLA_MPCCD_Cheetah_File(dials_data, lazy):
 
     format_class = dxtbx.format.Registry.get_format_class_for_file(filename)
 
-    iset = format_class.get_imageset([filename], lazy=lazy)
+    iset = format_class.get_imageset([filename], format_kwargs={"lazy": lazy})
 
     assert len(iset) == 4
     for i in range(len(iset)):
@@ -512,7 +518,9 @@ def test_SACLA_MPCCD_Cheetah_File(dials_data, lazy):
         assert iset.get_goniometer(i) is None
         assert iset.get_scan(i) is None
 
-    iset = format_class.get_imageset([filename], single_file_indices=[1], lazy=lazy)
+    iset = format_class.get_imageset(
+        [filename], single_file_indices=[1], format_kwargs={"lazy": lazy}
+    )
     assert len(iset) == 1
 
     for i in range(len(iset)):
